// ----------------------------------------------------------------------------
// Copyright (c) 2014, Nicolas P. Rougier. All Rights Reserved.
// Distributed under the (new) BSD License.
// ----------------------------------------------------------------------------
#include "colormaps/util.glsl"
#include "colormaps/user.glsl"

#include "colormaps/hot.glsl"
#include "colormaps/gray.glsl"
#include "colormaps/cool.glsl"
#include "colormaps/wheel.glsl"

#include "colormaps/autumn.glsl"
#include "colormaps/winter.glsl"
#include "colormaps/spring.glsl"
#include "colormaps/summer.glsl"

#include "colormaps/ice.glsl"
#include "colormaps/fire.glsl"
#include "colormaps/icefire.glsl"

#include "colormaps/reds.glsl"
#include "colormaps/blues.glsl"
#include "colormaps/greens.glsl"
